## From: Lance Bachmeier <lanceb@ksu.edu>
## To: edd@debian.org
## Subject: Favourite Rcpp examples?
## Date: Fri, 22 Apr 2011 09:25:37 -0500

## Dirk,

## The following code does a simulation from a VAR(1) model. I have used it
## to demonstrate the use and benefits of Rcpp to my graduate students, who
## have limited knowledge of C++.

## It may not be perfect code but it demonstrates many of the basic ideas
## in 9 lines of C++. On my machine the benchmark says the R version takes
## 62 times as long to run. Feel free to use it if you find it helpful.

## Best regards,
## Lance


## License: GPL 2 or higher
##require(Rcpp)
##require(RcppArmadillo)
require(inline)

code <- '
  arma::mat coeff = Rcpp::as<arma::mat>(a);
  arma::mat errors = Rcpp::as<arma::mat>(e);
  int m = errors.n_rows; int n = errors.n_cols;
  arma::mat simdata(m,n);
  simdata.row(0) = arma::zeros<arma::mat>(1,n);
  for (int row=1; row<m; row++) {
    simdata.row(row) = simdata.row(row-1)*trans(coeff)+errors.row(row);
  }
  return Rcpp::wrap(simdata);
'

rcppSim <- cxxfunction(signature(a="numeric",e="numeric"),
                       code,plugin="RcppArmadillo")

a <- matrix(c(0.5,0.1,0.1,0.5),nrow=2)
e <- matrix(rnorm(10000),ncol=2)
rcppData <- rcppSim(a,e)

rSim <- function(coeff,errors) {
  simdata <- matrix(0,nrow(errors),ncol(errors))
  for (row in 2:nrow(errors)) {
    simdata[row,] = coeff %*% simdata[(row-1),] + errors[row,]
  }
  return(simdata)
}
rData <- rSim(a,e)
stopifnot(all.equal(rcppData, rData))

require(compiler)
compRsim <- cmpfun(rSim)

compRData <- compRsim(a,e)
stopifnot(all.equal(rcppData, compRData))

library(rbenchmark)
benchmark(rcppSim(a,e),
          rSim(a,e),
          compRsim(a,e),
          order="relative")


## --
## Lance Bachmeier
## Associate Professor
## Department of Economics
## Kansas State University
## Manhattan, KS 66506
## 785-532-4578
